@@ -138,6 +138,8 @@ struct syncbarrier {
 	pthread_cond_t      cond;  /* waiting non-synctasks */
 	struct list_head    waitq; /* waiting synctasks */
 	int                 count; /* count the number of wakes */
+        int                 waitfor; /* no. of wakes until which task can be in
+                                        waitq before being woken up. */
 };
 typedef struct syncbarrier syncbarrier_t;
 
@@ -229,7 +231,7 @@ struct syncopctx {
 	} while (0)
 
 
-#define SYNCOP(subvol, stb, cbk, op, params ...) do {                   \
+#define SYNCOP(subvol, stb, cbk, fn_op, params ...) do {                \
                 struct  synctask        *task = NULL;                   \
                 call_frame_t            *frame = NULL;                  \
                                                                         \
@@ -247,8 +249,9 @@ struct syncopctx {
                                                                         \
                 __yawn (stb);                                           \
                                                                         \
+                frame->op = get_fop_index_from_fn (subvol, fn_op);      \
                 STACK_WIND_COOKIE (frame, cbk, (void *)stb, subvol,     \
-                                   op, params);                         \
+                                   fn_op, params);                      \
                                                                         \
                 __yield (stb);                                          \
                 if (task)                                               \
@@ -258,6 +261,29 @@ struct syncopctx {
         } while (0)
 
 
+/*
+ * syncop_xxx() calls are executed in two ways, one is inside a synctask where
+ * the executing function will do 'swapcontext' and the other is without
+ * synctask where the executing thread is made to wait using pthread_cond_wait.
+ * Executing thread may change when syncop_xxx() is executed inside a synctask.
+ * This leads to errno_location change i.e. errno may give errno of
+ * non-executing thread. So errno is not touched inside a synctask execution.
+ * All gfapi calls are executed using the second way of executing syncop_xxx()
+ * where the executing thread waits using pthread_cond_wait so it is ok to set
+ * errno in these cases. The following macro makes syncop_xxx() behave just
+ * like a system call, where -1 is returned and errno is set when a failure
+ * occurs.
+ */
+#define DECODE_SYNCOP_ERR(ret) do {  \
+        if (ret < 0) {          \
+                errno = -ret;   \
+                ret = -1;       \
+        } else {                \
+                errno = 0;      \
+        }                       \
+        } while (0)
+
+
 #define SYNCENV_DEFAULT_STACKSIZE (2 * 1024 * 1024)
 
 struct syncenv * syncenv_new (size_t stacksize, int procmin, int procmax);
@@ -537,4 +563,20 @@ syncop_setactivelk (xlator_t *subvol, loc_t *loc,
                      lock_migration_info_t *locklist,  dict_t *xdata_in,
                      dict_t **xdata_out);
 
+int
+syncop_put (xlator_t *subvol, loc_t *loc, mode_t mode, mode_t umask,
+            uint32_t flags, struct iovec *vector, int32_t count,
+            off_t offset, struct iobref *iobref, dict_t *xattr,
+            struct iatt *iatt, dict_t *xdata_in, dict_t **xdata_out);
+
+int
+syncop_setactivelk_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                        int32_t op_ret, int32_t op_errno, dict_t *xdata);
+
+int
+syncop_icreate (xlator_t *subvol, loc_t *loc, mode_t mode, dict_t *xdata_out);
+
+int
+syncop_namelink (xlator_t *subvol, loc_t *loc, dict_t *xdata_out);
+
 #endif /* _SYNCOP_H */
